﻿using System;

namespace SourceStat.Utils
{
	// *************************************************************************
	// *** FileSizeProvider ****************************************************
	// *************************************************************************
	public class FileSizeProvider : object, ICustomFormatter, IFormatProvider
	{
		// ---------------------------------------------------------------------
		public const string TypeAuto = "AUTO";	// Automatic recalculation onto the highest supported unit
		// ---------------------------------------------------------------------
		public static readonly FileSizeProvider MyFileSizeProvider = new FileSizeProvider();

		// ---------------------------------------------------------------------
		// --- FileSizeProvider Enums ------------------------------------------
		// ---------------------------------------------------------------------
		private enum Unit
		{
			B,
			KB,
			MB,
			GB,
			TB
		} // Unit

		// ---------------------------------------------------------------------
		// --- FileSizeProvider Methods ----------------------------------------
		// ---------------------------------------------------------------------
		public string Format(string formatString, object arg, IFormatProvider formatProvide)
		{
			// Invalid input
			if (formatString == null || (!(arg is long) && !(arg is int)))
			{
				if (arg as IFormattable != null)
					return (arg as IFormattable).ToString(formatString, formatProvide);
				
				return arg.ToString();
			} // if

			long value;
			if (arg is int)
				value = (long)((int)arg);
			else
				value = (long)arg;

			Unit useUnit = Unit.TB;
			double conversionValue;
			foreach (Unit unit in Enum.GetValues(typeof(Unit)))
			{
				conversionValue = Math.Pow(1024.0, (double)((int)unit));
				
				if (value / conversionValue < 1.0)
				{
					if (unit == Unit.B)
						useUnit = unit;
					else
						useUnit = (Unit)(((int)unit) - 1); // Prev unit
					break;
				} // if
			} // foreach

			return string.Format("{0:#,0.00} {1}", value / Math.Pow(1024.0, (double)((int)useUnit)), useUnit.ToString());
		}

		// ---------------------------------------------------------------------
		public object GetFormat(Type formatType)
		{
			if (typeof(ICustomFormatter).Equals(formatType))
				return this;

			return null;
		}
	} // public class FileSizeProvider

} // namespace SourceStat.Utils