﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace SourceStat.Utils
{
	public static class StringUtils
	{
		// ---------------------------------------------------------------------
		// --- StringUtils Methods ---------------------------------------------
		// ---------------------------------------------------------------------
		/// <summary>
		/// Faster string.IndexOf() for finding commentars		
		/// http://www.codeproject.com/KB/string/string_optimizations.aspx
		/// </summary>
		/// <param name="source"></param>
		/// <param name="pattern"></param>
		/// <returns></returns>
		public static int InvariantCultureIndexOf(string source, string pattern/*???, int startIndex*/)
		{
			if (pattern == null)
				throw new ArgumentNullException();
			
			if (pattern.Length == 0)
				return 0;
			
			if (pattern.Length == 1)
				return source.IndexOf(pattern[0]);
			
			bool found;
			int limit = source.Length - pattern.Length + 1;
			if (limit < 1)
				return -1;
			
			// Store the first 2 characters of "pattern"
			char c0 = pattern[0];
			char c1 = pattern[1];
			
			// Find the first occurrence of the first character
			int first = source.IndexOf(c0, 0, limit);
			while (first != -1)
			{
				// Check if the following character is the same like
				// the 2nd character of "pattern"
				if (source[first + 1] != c1)
				{
					first = source.IndexOf(c0, ++first, limit - first);
					continue;
				}
				// Check the rest of "pattern" (starting with the 3rd character)
				found = true;
				for (int j = 2; j < pattern.Length; j++)
				{
					if (source[first + j] != pattern[j])
					{
						found = false;
						break;
					}
				} // for
				// If the whole word was found, return its index, otherwise try again
				if (found) return first;
				first = source.IndexOf(c0, ++first, limit - first);
			} // while

			return -1;
		}
	} // public static class StringUtils

} // namespace SourceStat.Utils