﻿using System;
using System.Diagnostics;
using System.Reflection;
using System.Windows.Forms;
using SourceStat.Analysis;
using SourceStat.Utils;

namespace SourceStat
{
	// *************************************************************************
	// *** ProtocolResultHelper ************************************************
	// *************************************************************************
	public static class ProtocolResultHelper
	{
		// ---------------------------------------------------------------------
		// --- ProtocolResultHelper Methods ------------------------------------
		// ---------------------------------------------------------------------
		public static void InitActionsList(CheckedListBox checkedListBoxActions)
		{
			bool isValidInput = (checkedListBoxActions != null);
			Debug.Assert(isValidInput, AssertUtils.Null);
			if (!isValidInput)
				return;

			checkedListBoxActions.Items.Clear();

			ProtocolResult result = new ProtocolResult();
			foreach (FieldInfo field in result.GetFields())
			{
				ActionAttribute[] descriptionAttribute = (ActionAttribute[])field.GetCustomAttributes(typeof(ActionAttribute), /*inherited*/true);
				if (descriptionAttribute.Length > 0)
					checkedListBoxActions.Items.Add(descriptionAttribute[0].Description, /*isChecked*/(field.GetValue(result) != null));
			} // foreach
		}

		// ---------------------------------------------------------------------
		public static ProtocolResult GetProtocolResult(CheckedListBox checkedListBoxActions)
		{
			bool isValidInput = (checkedListBoxActions != null);
			Debug.Assert(isValidInput, AssertUtils.Null);
			if (!isValidInput)
				return new ProtocolResult();

			ProtocolResult result = new ProtocolResult();
			foreach (FieldInfo field in result.GetFields())
			{
				ActionAttribute[] descriptionAttribute = (ActionAttribute[])field.GetCustomAttributes(typeof(ActionAttribute), /*inherited*/true);
				if (descriptionAttribute.Length > 0)
				{
					// Match item with description
					int itemIndex = checkedListBoxActions.FindString(descriptionAttribute[0].Description);
					bool stringFound = (checkedListBoxActions.FindString(descriptionAttribute[0].Description) >= 0);
					Debug.Assert(stringFound, "Cannot match checkboxlist itme with result field!");

					if (stringFound)
					{
						object value;
						if (field.FieldType.Equals(typeof(long?)))
							value = (checkedListBoxActions.CheckedIndices.Contains(itemIndex) ? 0 : (long?)null);
						else
							throw new NotSupportedException();

						field.SetValue(result, value);
					} // if
				} // if
			} // foreach

			return result;
		}

		// ---------------------------------------------------------------------
		public static void DisplayResult(ListBox.ObjectCollection items, ProtocolResult result)
		{
			bool isValidInput = (items != null && result != null);
			Debug.Assert(isValidInput, AssertUtils.Null);
			if (!isValidInput)
				return;

			foreach (FieldInfo field in result.GetFields())
			{
				ActionAttribute[] descriptionAttribute = (ActionAttribute[])field.GetCustomAttributes(typeof(ActionAttribute), /*inherited*/true);
				if (descriptionAttribute.Length > 0 && field.GetValue(result) != null)
				{
					string value;
					if (descriptionAttribute[0].FormatType == ActionAttribute.FormatTypes.FileSize)
						value = string.Format(FileSizeProvider.MyFileSizeProvider, "{0:AUTO}", field.GetValue(result));
					else
						value = string.Format("{0:#,0}", field.GetValue(result));

					items.Add(descriptionAttribute[0].Description + ": " + value);
				} // if
			} // foreach
		}
	} // public static class ProtocolResultHelper

} // namespace SourceStat